@@ -317,6 +317,22 @@ struct _SortFuncData {
 };
 typedef struct _SortFuncData SortFuncData;
 
+static MuContainer*
+get_top_msg (MuContainer *c, MuMsgFieldId mfid)
+{
+  MuContainer *piv, *extreme = c;
+  for (piv = c; piv != NULL && piv->msg != NULL; piv = piv->child) {
+    if (mu_msg_cmp (piv->msg, extreme->msg, mfid) > 0)
+      extreme = piv;
+    if (piv != c && piv->next) {
+      MuContainer *sub = get_top_msg (piv->next, mfid);
+
+      if (sub->msg != NULL && mu_msg_cmp (sub->msg, extreme->msg, mfid) > 0)
+        extreme = sub;
+    }
+  }
+  return extreme;
+}
 
 static int
 sort_func_wrapper (MuContainer *a, MuContainer *b, SortFuncData *data)
@@ -328,6 +344,9 @@ sort_func_wrapper (MuContainer *a, MuContainer *b, SortFuncData *data)
 	for (a1 = a; a1->msg == NULL && a1->child != NULL; a1 = a1->child);
 	for (b1 = b; b1->msg == NULL && b1->child != NULL; b1 = b1->child);
 
+  a1 = get_top_msg (a1, data->mfid);
+  b1 = get_top_msg (b1, data->mfid);
+
 	if (a1 == b1)
 		return 0;
 	else if (!a1->msg)
@@ -520,7 +539,7 @@ count_colons (const char *str)
 
 
 static MuMsgIterThreadInfo*
-thread_info_new (gchar *threadpath, gboolean root, gboolean child,
+thread_info_new (gchar *threadpath, gboolean root, gboolean first_child,
 		 gboolean empty_parent, gboolean has_child, gboolean is_dup)
 {
 	MuMsgIterThreadInfo *ti;
@@ -529,9 +548,9 @@ thread_info_new (gchar *threadpath, gboolean root, gboolean child,
 	ti->threadpath	     = threadpath;
 	ti->level            = count_colons (threadpath); /* hacky... */
 
-	ti->prop  = 0;
+	ti->prop  = MU_MSG_ITER_THREAD_PROP_NONE;
 	ti->prop |= root         ? MU_MSG_ITER_THREAD_PROP_ROOT         : 0;
-	ti->prop |= child        ? MU_MSG_ITER_THREAD_PROP_FIRST_CHILD  : 0;
+	ti->prop |= first_child  ? MU_MSG_ITER_THREAD_PROP_FIRST_CHILD  : 0;
 	ti->prop |= empty_parent ? MU_MSG_ITER_THREAD_PROP_EMPTY_PARENT : 0;
 	ti->prop |= is_dup       ? MU_MSG_ITER_THREAD_PROP_DUP          : 0;
 	ti->prop |= has_child    ? MU_MSG_ITER_THREAD_PROP_HAS_CHILD    : 0;
