@@ -22,12 +22,15 @@
 #include "config.h"
 #endif /*HAVE_CONFIG_H*/
 
+#define _BSD_SOURCE
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
 #include <stdarg.h>
+#include <sys/time.h>
 
 #include <glib/gprintf.h>
 
@@ -166,11 +169,11 @@ print_and_clear_g_error (GError **err)
 }
 
 
-static GSList*
-read_line_as_list (GError **err)
+static GHashTable*
+read_command_line (GError **err)
 {
 	char *line;
-	GSList *lst;
+	GHashTable *hash;
 	GString *gstr;
 
 	line = NULL;
@@ -190,43 +193,33 @@ read_line_as_list (GError **err)
 	} while (1);
 
 	line = g_string_free (gstr, FALSE);
-	lst = mu_str_esc_to_list (line, err);
+
+	if (!mu_str_is_empty (line))
+		hash = mu_str_parse_arglist (line, err);
+	else
+		hash = NULL;
 
 	g_free (line);
 
-	return lst;
+	return hash;
 }
 
 
 static const char*
-get_string_from_args (GSList *args, const char *param, gboolean optional,
+get_string_from_args (GHashTable *args, const char *param, gboolean optional,
 		      GError **err)
 {
-	size_t param_len;
+	const char *str;
 
-	param_len = strlen (param);
-
-	while (args) {
-
-		const char *arg;
-		arg = (const char*)args->data;
-
-		/* do we have this param */
-		if (arg && g_str_has_prefix (arg, param) &&
-		    arg[param_len] == ':')
-			return (const char*) arg + param_len + 1;
-
-		args = g_slist_next (args);
-	}
-
-	if (!optional)
+	str = g_hash_table_lookup (args, param);
+	if (!str && !optional)
 		mu_util_g_set_error (err, MU_ERROR_IN_PARAMETERS,
 				     "parameter '%s' not found", param);
-	return NULL;
+	return str;
 }
 
 static gboolean
-get_bool_from_args (GSList *args, const char *param, gboolean optional, GError **err)
+get_bool_from_args (GHashTable *args, const char *param, gboolean optional, GError **err)
 {
 	const char *val;
 
@@ -330,7 +323,7 @@ get_docids_from_msgids (MuQuery *query, const char *str, GError **err)
  * locale the message with message-id in the database, and return its
  * docid */
 static unsigned
-determine_docid (MuQuery *query, GSList *args, GError **err)
+determine_docid (MuQuery *query, GHashTable *args, GError **err)
 {
 	const char* docidstr, *msgidstr;
 
@@ -379,7 +372,7 @@ typedef struct _ServerContext ServerContext;
  * if function return non-MU_OK, the repl will print the error instead
  */
 
-typedef MuError (*CmdFunc) (ServerContext*,GSList*,GError**);
+typedef MuError (*CmdFunc) (ServerContext*,GHashTable*,GError**);
 
 /* 'add' adds a message to the database, and takes two parameters:
  * 'path', which is the full path to the message, and 'maildir', which
@@ -388,7 +381,7 @@ typedef MuError (*CmdFunc) (ServerContext*,GSList*,GError**);
  * message (details: see code below)
  */
 static MuError
-cmd_add (ServerContext *ctx, GSList *args, GError **err)
+cmd_add (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	unsigned docid;
 	const char *maildir, *path;
@@ -507,7 +500,7 @@ compose_type (const char *typestr)
  * Note ':include' t or nil determines whether to include attachments
  */
 static MuError
-cmd_compose (ServerContext *ctx, GSList *args, GError **err)
+cmd_compose (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	const gchar *typestr;
 	char *sexp, *atts;
@@ -557,7 +550,7 @@ typedef struct _SexpData SexpData;
 
 static void
 each_contact_sexp (const char *email, const char *name, gboolean personal,
-		   time_t tstamp, SexpData *sdata)
+		   time_t tstamp, unsigned freq, SexpData *sdata)
 {
 	char *escmail;
 
@@ -579,11 +572,14 @@ each_contact_sexp (const char *email, const char *name, gboolean personal,
 	if (name) {
 		char *escname;
 		escname = mu_str_escape_c_literal (name, TRUE);
-		g_string_append_printf (sdata->gstr, "(:name %s :mail %s)\n",
-					escname, escmail);
+		g_string_append_printf (sdata->gstr,
+					"(:name %s :mail %s :tstamp %u :freq %u)\n",
+					escname, escmail, (unsigned)tstamp, freq);
 		g_free (escname);
 	} else
-		g_string_append_printf (sdata->gstr, "(:mail %s)\n", escmail);
+		g_string_append_printf (sdata->gstr,
+					"(:mail %s :tstamp %u :freq %u)\n",
+					escmail, (unsigned)tstamp, freq);
 
 	g_free (escmail);
 }
@@ -620,7 +616,7 @@ contacts_to_sexp (MuContacts *contacts, gboolean personal, time_t after)
 
 
 static MuError
-cmd_contacts (ServerContext *ctx, GSList *args, GError **err)
+cmd_contacts (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	MuContacts *contacts;
 	char *sexp;
@@ -680,7 +676,7 @@ print_sexps (MuMsgIter *iter, unsigned maxnum)
 
 static MuError
 save_part (MuMsg *msg, unsigned docid,
-	   unsigned index, GSList *args, GError **err)
+	   unsigned index, GHashTable *args, GError **err)
 {
 	gboolean rv;
 	const gchar *path;
@@ -740,7 +736,7 @@ leave:
 
 
 static MuError
-temp_part (MuMsg *msg, unsigned docid, unsigned index, GSList *args,
+temp_part (MuMsg *msg, unsigned docid, unsigned index, GHashTable *args,
 	   GError **err)
 {
 	const char *what, *param;
@@ -798,13 +794,15 @@ action_type (const char *actionstr)
 
 /* 'extract' extracts some mime part from a message */
 static MuError
-cmd_extract (ServerContext *ctx, GSList *args, GError **err)
+cmd_extract (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	MuMsg *msg;
 	int docid, index, action;
 	MuError rv;
 	const char* actionstr, *indexstr;
 
+	rv = MU_ERROR;
+
 	/* read parameters */
 	GET_STRING_OR_ERROR_RETURN (args, "action", &actionstr, err);
 	GET_STRING_OR_ERROR_RETURN (args, "index",  &indexstr, err);
@@ -839,15 +837,17 @@ cmd_extract (ServerContext *ctx, GSList *args, GError **err)
 	return MU_OK;
 }
 
+#define MAX_NUM_DEFAULT 500
+
 /* parse the find parameters, and return the values as out params */
 static MuError
-get_find_params (GSList *args, MuMsgFieldId *sortfield,
+get_find_params (GHashTable *args, MuMsgFieldId *sortfield,
 		 int *maxnum, MuQueryFlags *qflags, GError **err)
 {
 	const char *maxnumstr, *sortfieldstr;
 
 	/* defaults */
-	*maxnum	   = 500;
+	*maxnum	   = MAX_NUM_DEFAULT;
 	*qflags	   = MU_QUERY_FLAG_NONE;
 	*sortfield = MU_MSG_FIELD_ID_NONE;
 
@@ -868,7 +868,8 @@ get_find_params (GSList *args, MuMsgFieldId *sortfield,
 
 	/* maximum number of results */
 	maxnumstr = get_string_from_args (args, "maxnum", TRUE, NULL);
-	*maxnum = maxnumstr ? atoi (maxnumstr) : 0;
+	if (maxnumstr)
+		*maxnum = atoi (maxnumstr);
 
 	if (get_bool_from_args (args, "reverse", TRUE, NULL))
 		*qflags |= MU_QUERY_FLAG_DESCENDING;
@@ -895,7 +896,7 @@ get_find_params (GSList *args, MuMsgFieldId *sortfield,
  * (:found <number of found messages>)
  */
 static MuError
-cmd_find (ServerContext *ctx, GSList *args, GError **err)
+cmd_find (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	MuMsgIter *iter;
 	unsigned foundnum;
@@ -947,7 +948,7 @@ cmd_find (ServerContext *ctx, GSList *args, GError **err)
 
 #ifdef BUILD_GUILE
 static MuError
-cmd_guile (ServerContext *ctx, GSList *args, GError **err)
+cmd_guile (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	const char *script, *file;
 
@@ -964,7 +965,7 @@ cmd_guile (ServerContext *ctx, GSList *args, GError **err)
 }
 #else /*!BUILD_GUILE*/
 static MuError
-cmd_guile (ServerContext *ctx, GSList *args, GError **err)
+cmd_guile (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	print_error (MU_ERROR_INTERNAL,
 		     "this mu does not have guile support");
@@ -1061,7 +1062,7 @@ index_and_cleanup (MuIndex *index, const char *path, GError **err)
  * index ... ) messages while doing so (see the code)
  */
 static MuError
-cmd_index (ServerContext *ctx, GSList *args, GError **err)
+cmd_index (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	MuIndex *index;
 	const char *argpath;
@@ -1097,7 +1098,7 @@ leave:
 /* 'mkdir' attempts to create a maildir directory at 'path:'; sends an
  * (:info mkdir ...) message when it succeeds */
 static MuError
-cmd_mkdir (ServerContext *ctx, GSList *args, GError **err)
+cmd_mkdir (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	const char *path;
 
@@ -1136,7 +1137,7 @@ get_flags (const char *path, const char *flagstr)
 
 static MuError
 do_move (MuStore *store, unsigned docid, MuMsg *msg, const char *maildir,
-	 MuFlags flags, GError **err)
+	 MuFlags flags, gboolean new_name, GError **err)
 {
 	unsigned rv;
 	gchar *sexp;
@@ -1150,12 +1151,12 @@ do_move (MuStore *store, unsigned docid, MuMsg *msg, const char *maildir,
 		different_mdir =
 			(g_strcmp0 (maildir, mu_msg_get_maildir(msg)) != 0);
 
-	if (!mu_msg_move_to_maildir (msg, maildir, flags, TRUE, err))
+	if (!mu_msg_move_to_maildir (msg, maildir, flags, TRUE,
+				     new_name, err))
 		return MU_G_ERROR_CODE (err);
 
-	/* note, after mu_msg_move_to_maildir, path will be the *new*
-	 * path, and flags and maildir fields will be updated as
-	 * wel */
+	/* after mu_msg_move_to_maildir, path will be the *new* path,
+	 * and flags and maildir fields will be updated as wel */
 	rv = mu_store_update_msg (store, docid, msg, err);
 	if (rv == MU_STORE_INVALID_DOCID) {
 		mu_util_g_set_error (err, MU_ERROR_XAPIAN,
@@ -1174,11 +1175,12 @@ do_move (MuStore *store, unsigned docid, MuMsg *msg, const char *maildir,
 }
 
 static MuError
-move_msgid (MuStore *store, unsigned docid, const char* flagstr, GError **err)
+move_docid (MuStore *store, unsigned docid, const char* flagstr,
+	    gboolean new_name, GError **err)
 {
-	MuMsg *msg;
-	MuError rv;
-	MuFlags flags;
+	MuMsg		*msg;
+	MuError		 rv;
+	MuFlags		 flags;
 
 	rv  = MU_ERROR;
 	msg = mu_store_get_msg (store, docid, err);
@@ -1188,13 +1190,14 @@ move_msgid (MuStore *store, unsigned docid, const char* flagstr, GError **err)
 
 	flags = flagstr ? get_flags (mu_msg_get_path(msg), flagstr) :
 		mu_msg_get_flags (msg);
+
 	if (flags == MU_FLAG_INVALID) {
 		mu_util_g_set_error (err, MU_ERROR_IN_PARAMETERS,
 				     "invalid flags");
 		goto leave;
 	}
 
-	rv = do_move (store, docid, msg, NULL, flags, err);
+	rv = do_move (store, docid, msg, NULL, flags, new_name, err);
 
 leave:
 	if (msg)
@@ -1207,17 +1210,21 @@ leave:
 
 
 /* when called with a msgid, we need to take care of possibly multiple
- * message with this message id. this is a common case when sending
+ * messages with this message id. this is a common case when sending
  * messages to ourselves (maybe through a mailing list), where there
- * would a message in inbox and sentbox with the same id. we set the
- * flag on both */
+ * would a message in the inbox and sent folders with the same id. we
+ * set the flag on both */
 static gboolean
-move_msgid_maybe (ServerContext *ctx, GSList *args, GError **err)
+move_msgid_maybe (ServerContext *ctx, GHashTable *args, GError **err)
 {
-	GSList *docids, *cur;
-	const char *maildir = get_string_from_args (args, "maildir", TRUE, err);
-	const char *msgid   = get_string_from_args (args, "msgid", TRUE, err);
-	const char *flagstr = get_string_from_args (args, "flags", TRUE, err);
+	GSList		*docids, *cur;
+	const char*	 maildir, *msgid, *flagstr;
+	gboolean	 new_name;
+
+	maildir	 = get_string_from_args (args, "maildir", TRUE, err);
+	msgid	 = get_string_from_args (args, "msgid", TRUE, err);
+	flagstr	 = get_string_from_args (args, "flags", TRUE, err);
+	new_name = get_bool_from_args (args, "newname", TRUE, err);
 
 	/*  you cannot use 'maildir' for multiple messages at once */
 	if (!msgid || !flagstr || maildir)
@@ -1229,8 +1236,8 @@ move_msgid_maybe (ServerContext *ctx, GSList *args, GError **err)
 	}
 
 	for (cur = docids; cur; cur = g_slist_next(cur))
-		if (move_msgid (ctx->store, GPOINTER_TO_SIZE(cur->data),
-				 flagstr, err) != MU_OK)
+		if (move_docid (ctx->store, GPOINTER_TO_SIZE(cur->data),
+				flagstr, new_name, err) != MU_OK)
 			break;
 
 	g_slist_free (docids);
@@ -1250,20 +1257,22 @@ move_msgid_maybe (ServerContext *ctx, GSList *args, GError **err)
  *
  */
 static MuError
-cmd_move (ServerContext *ctx, GSList *args, GError **err)
+cmd_move (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	unsigned docid;
 	MuMsg *msg;
 	MuFlags flags;
 	const char *maildir, *flagstr;
+	gboolean new_name;
 
 	/* check if the move is based on the message id; if so, handle
 	 * it in move_msgid_maybe */
 	if (move_msgid_maybe (ctx, args, err))
 		return MU_OK;
 
-	maildir	= get_string_from_args (args, "maildir", TRUE, err);
-	flagstr = get_string_from_args (args, "flags", TRUE, err);
+	maildir	 = get_string_from_args (args, "maildir", TRUE, err);
+	flagstr	 = get_string_from_args (args, "flags", TRUE, err);
+	new_name = get_bool_from_args (args, "newname", TRUE, err);
 
 	docid = determine_docid (ctx->query, args, err);
 	if (docid == MU_STORE_INVALID_DOCID ||
@@ -1289,7 +1298,8 @@ cmd_move (ServerContext *ctx, GSList *args, GError **err)
 		goto leave;
 	}
 
-	if ((do_move (ctx->store, docid, msg, maildir, flags, err) != MU_OK))
+	if ((do_move (ctx->store, docid, msg, maildir, flags, new_name, err)
+	     != MU_OK))
 		print_and_clear_g_error (err);
 
 leave:
@@ -1303,7 +1313,7 @@ leave:
  * server using a (:pong ...) message (details: see code below)
  */
 static MuError
-cmd_ping (ServerContext *ctx, GSList *args, GError **err)
+cmd_ping (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	unsigned doccount;
 	doccount = mu_store_count (ctx->store, err);
@@ -1326,7 +1336,7 @@ cmd_ping (ServerContext *ctx, GSList *args, GError **err)
 
 /* 'quit' takes no parameters, terminates this mu server */
 static MuError
-cmd_quit (ServerContext *ctx, GSList *args , GError **err)
+cmd_quit (ServerContext *ctx, GHashTable *args , GError **err)
 {
 	print_expr (";; quiting");
 
@@ -1370,7 +1380,7 @@ get_path_from_docid (MuStore *store, unsigned docid, GError **err)
  * (:remove ...) message when it succeeds
  */
 static MuError
-cmd_remove (ServerContext *ctx, GSList *args, GError **err)
+cmd_remove (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	unsigned docid;
 	const char *path;
@@ -1411,7 +1421,7 @@ cmd_remove (ServerContext *ctx, GSList *args, GError **err)
  * message (details: see code below)
  */
 static MuError
-cmd_sent (ServerContext *ctx, GSList *args, GError **err)
+cmd_sent (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	unsigned docid;
 	const char *maildir, *path;
@@ -1434,7 +1444,7 @@ cmd_sent (ServerContext *ctx, GSList *args, GError **err)
 }
 
 static MuMsgOptions
-get_view_msg_opts (GSList *args)
+get_view_msg_opts (GHashTable *args)
 {
 	MuMsgOptions opts;
 
@@ -1456,7 +1466,7 @@ get_view_msg_opts (GSList *args)
  * identified by either docid: or msgid:; return a (:view <sexp>)
  */
 static MuError
-cmd_view (ServerContext *ctx, GSList *args, GError **err)
+cmd_view (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	MuMsg *msg;
 	const gchar *path;
@@ -1502,7 +1512,7 @@ cmd_view (ServerContext *ctx, GSList *args, GError **err)
 /*************************************************************************/
 
 static MuError
-handle_args (ServerContext *ctx, GSList *args, GError **err)
+handle_args (ServerContext *ctx, GHashTable *args, GError **err)
 {
 	unsigned u;
 	const char *cmd;
@@ -1526,7 +1536,7 @@ handle_args (ServerContext *ctx, GSList *args, GError **err)
 		{ "view",	cmd_view }
 	};
 
-	cmd = (const char*) args->data;
+	cmd = g_hash_table_lookup (args, "cmd");
 
 	/* ignore empty */
 	if (mu_str_is_empty (cmd))
@@ -1534,7 +1544,7 @@ handle_args (ServerContext *ctx, GSList *args, GError **err)
 
 	for (u = 0; u != G_N_ELEMENTS (cmd_map); ++u)
 		if (g_strcmp0(cmd, cmd_map[u].cmd) == 0)
-			return cmd_map[u].func (ctx, g_slist_next(args), err);
+			return cmd_map[u].func (ctx, args, err);
 
 	mu_util_g_set_error (err, MU_ERROR_IN_PARAMETERS,
 			     "unknown command '%s'", cmd ? cmd : "");
@@ -1557,6 +1567,8 @@ mu_cmd_server (MuStore *store, MuConfig *opts/*unused*/, GError **err)
 	if (!ctx.query)
 		return MU_G_ERROR_CODE (err);
 
+	srand (time(NULL)*getpid());
+
 	install_sig_handler ();
 
 	g_print (";; welcome to " PACKAGE_STRING "\n");
@@ -1565,13 +1577,18 @@ mu_cmd_server (MuStore *store, MuConfig *opts/*unused*/, GError **err)
 	do_quit = FALSE;
 	while (!MU_TERMINATE && !do_quit) {
 
-		GSList *args;
-		GError *my_err = NULL;
+		GHashTable	*args;
+		GError		*my_err;
 
 		/* args will receive a the command as a list of
 		 * strings. returning NULL indicates an error */
-		args   = read_line_as_list (&my_err);
-		if (!args || my_err) {
+		my_err = NULL;
+		args   = read_command_line (&my_err);
+		if ((!args || g_hash_table_size(args) == 0) && !my_err) {
+			if (args)
+				g_hash_table_destroy (args);
+			continue;
+		} else if (my_err) {
 			print_and_clear_g_error (&my_err);
 			continue;
 		}
@@ -1585,7 +1602,7 @@ mu_cmd_server (MuStore *store, MuConfig *opts/*unused*/, GError **err)
 			print_and_clear_g_error (&my_err);
 		}
 
-		mu_str_free_list (args);
+		g_hash_table_destroy (args);
 	}
 
 	mu_store_flush   (ctx.store);
